from ftw import ruleset, logchecker, testrunner
import datetime
import pytest
import sys
import re
import os


def test_crs(ruleset, test, logchecker_obj):
    runner = testrunner.TestRunner()
    for stage in test.stages:
        runner.run_stage(stage, logchecker_obj)


class FooLogChecker(logchecker.LogChecker):
    def __init__(self, config):
        super(FooLogChecker, self).__init__()
        self.log_location = config['log_location_linux']
        self.log_date_regex = config['log_date_regex']
        self.log_date_format = config['log_date_format']

    def reverse_readline(self, filename):
        with open(filename) as f:
            f.seek(0, os.SEEK_END)
            position = f.tell()
            line = ''
            while position >= 0:
                f.seek(position)
                next_char = f.read(1)
                if next_char == "\n":
                    yield line[::-1]
                    line = ''
                else:
                    line += next_char
                position -= 1
            yield line[::-1]

    def get_logs(self):
        pattern = re.compile(r'%s' % self.log_date_regex)
        our_logs = []
        for lline in self.reverse_readline(self.log_location):
            # Extract dates from each line
            match = re.match(pattern, lline)
            if match:
                log_date = match.group(1)
                log_date = datetime.datetime.strptime(
                    log_date, self.log_date_format)
                # NGINX doesn't give us microsecond level by detail, round down.
                if "%f" not in self.log_date_format:
                    ftw_start = self.start.replace(microsecond=0)
                else:
                    ftw_start = self.start
                ftw_end = self.end
                if log_date <= ftw_end and log_date >= ftw_start:
                    our_logs.append(lline)
                # If our log is from before FTW started stop
                if log_date < ftw_start:
                    break
        return our_logs


@pytest.fixture(scope='session')
def logchecker_obj(config):
    return FooLogChecker(config)



#!/usr/bin/env python
#@spartantri 2018

import sys
import argparse
import base64

parser = argparse.ArgumentParser()

UserAgent="ModSecurity CRS 3 Tests"
Accept="text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5"
AcceptCharset="ISO-8859-1,utf-8;q=0.7,*;q=0.7"
AcceptEncoding="gzip,deflate"
AcceptLanguage="en-us,en;q=0.5"
ContentType="application/x-www-form-urlencoded"
payloads=[]
skeletontest=0
Meta='''---
  meta:
    author: "spartantri"
    enabled: true
    name: "skeletonid.yaml"
    description: "Positive tests for rule skeletonid"
  tests:
'''

parser.add_argument('-a', action='store', dest='Addr', help='Target ip address',
                     default='127.0.0.1')
parser.add_argument('-p', action='store', dest='Port', help='Target port',
                     default='80')
parser.add_argument('-v', action='store', dest='Host', help='Target virtual host',
                     default='localhost')
parser.add_argument('-s', action='store', dest='skeleton', help='Skeleton file',
                     default='positivetest.yaml.skeleton')
parser.add_argument('-o', action='store', dest='output', help='output file',
                     default='')
parser.add_argument('-r', action='store', dest='ruleid', help='Rule id',
                     default='944310')
parser.add_argument('-k', action='store', dest='combined_payload', help='Keyword containing combined pipe separated payloads',
                     default='')
parser.add_argument('-i', action='append', dest='list_payload', help='Keyword containing individual payload',
                     default=[])
parser.add_argument('-c', action='store', dest='prefix', help='Prefix keyword for all payloads',
                     default='')
parser.add_argument('-e', action='store', dest='sufix', help='Sufix keyword for all payloads',
                     default='')
parser.add_argument('-b', action='store_true', dest='base64encode', help='Encode payload using Base64',
                     default=False)
parser.add_argument('-d', action='store_true', dest='demo', help='Print demo rules if no data is provided',
                     default=False)
parser.add_argument('-t', action='store_true', dest='test', help='Launch FTW and test output',
                     default=False)
parser.add_argument('-w', action='store', dest='author', help='Test author',
                     default='spartantri')
start_options = parser.parse_args()

if len(sys.argv)<2 and not start_options.demo:
    parser.print_usage()
    exit()

for p in start_options.combined_payload.split('|'):
    if p not in payloads:
        # print('Checking %s' % (p))
        if len(p)>0:
            payloads.append(''.join([start_options.prefix, p, start_options.sufix]))

for p in start_options.list_payload:
    payloads.append(''.join([start_options.prefix, p, start_options.sufix]))

if start_options.output=='':
    o=sys.stdout
else:
    o=open(start_options.output, 'w')

o.write(Meta.replace('skeletonid', start_options.ruleid).replace('spartantri', start_options.author))
for item in payloads:
    if start_options.base64encode:
        payload=base64.encodestring(item).replace('\n', '')
        #print payload
    else:
        payload=item
    with open(start_options.skeleton,'r') as f:
        for l in f:
            l=l.replace('skeletonid', start_options.ruleid)
            l=l.replace('skeletonkeyword', payload)
            l=l.replace('skeletondefaultaddr', start_options.Addr)
            l=l.replace('skeletondefaultport', start_options.Port)
            l=l.replace('skeletondefaulthost', start_options.Host)
            l=l.replace('skeletondefaultuseragent', UserAgent)
            l=l.replace('skeletondefaultacceptcharset', AcceptCharset)
            l=l.replace('skeletondefaultacceptencoding', AcceptEncoding)
            l=l.replace('skeletondefaultacceptlanguage', AcceptLanguage)
            l=l.replace('skeletondefaultcontenttype', ContentType)
            l=l.replace('skeletondefaultaccept', Accept)
            if 'skeletontest' in l:
                l=l.replace('skeletontest', str(skeletontest))
                skeletontest+=1
            o.write(l)

if start_options.output != '':
    print ('Generated %s rules to file %s' % (str(skeletontest), start_options.output))
    o.close()

#print('\nGenerated %s tests' % (str(skeletontest)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""This is designed to convert 2.x CRS ID numbering to 3.x CRS numbering"""
from __future__ import print_function
import csv
import argparse
import os
import sys

def main():
    """Main function that contains all the logic to relabel CRS IDs"""

    id_translation_file = os.path.join(sys.path[0], "IdNumbering.csv")

    if not os.path.isfile(id_translation_file):
        sys.stderr.write("We were unable to locate the ID translation CSV (idNumbering.csv) \
            please place this is the same directory as this script\n")
        sys.exit(1)

    parser = argparse.ArgumentParser(description="A program that takes in an exceptions file \
        and renumbers all the ID to match OWASP CRS 3 numbers. Output will be directed to STDOUT.")
    parser.add_argument("-f", "--file", required=True, action="store", dest="fname", \
        help="the file to be renumbered")
    args = parser.parse_args()

    if not os.path.isfile((args.fname).encode('utf8')):
        sys.stderr.write("We were unable to find the file you were trying to update the ID numbers \
            in, please check your path\n")
        sys.exit(1)

    fcontent = ""

    try:
        update_file = open((args.fname).encode('utf-8'), "r")
        try:
            fcontent = update_file.read()
        finally:
            update_file.close()
    except IOError:
        sys.stderr.write("There was an error opening the file you were trying to update")

    if fcontent != "":
        # CSV File
        id_csv_file = open(id_translation_file, 'rt')
        try:
            reader = csv.reader(id_csv_file)
            for row in reader:
                fcontent = fcontent.replace(row[0], row[1])
        finally:
            id_csv_file.close()
    print(fcontent)

if __name__ == "__main__":
    main()



from ftw import ruleset, http, errors

"""
This script reads in a list of popular Useragents and checks to see if it triggers
It expects 403's to be returned for a rule firing
"""

def read_useragents(filename):
    f = open(filename,'r')
    useragents = [agent.strip() for agent in f.readlines()]
    return useragents

def run_requests(useragent_list):
    status_not_403 = 0
    status_403 = 0
    for useragent in useragent_list:
        # get me a counter while i'm waiting
        if (status_not_403 + status_403)%15 == 0:
            print("Send",status_not_403 + status_403, "Out of",len(useragent_list))
        input_data = ruleset.Input(method="GET", protocol="http",port=80,uri='/',dest_addr="localhost",headers={"Host":"localhost","User-Agent":useragent})
        http_ua = http.HttpUA()
        http_ua.send_request(input_data)
        status = http_ua.response_object.status
        if status == 403:	
            status_403 += 1
        else:
            status_not_403 += 1        
    x = (status_403/(len(useragent_list)*1.0))*100
    y = (status_not_403/(len(useragent_list)*1.0))*100
    print "403s =", x
    print "not 403s =", y

	
def main():
    uas = read_useragents('./data/popularUAs.data')
    run_requests(uas)
main()	



from ftw import ruleset, http, errors

"""
This script assumes that default blocking action is 403
and sampling is one. It will send a know bad request 
that is expected to be blocked. If sampling is on it
will only block a certain percentage. We send 1000 
requests to verify this. In order to do this we must
also turn off IP Reputation blocking.
SecAction "id:900005,phase:1,nolog,pass,ctl:ruleEngine=on,ctl:ruleRemoveById=910000"
"""
def send_requests(input_data,subiters,result,index):
	http_ua = http.HttpUA()
	for i in range(0,subiters):
		new_index = str(index)+str(i)
		http_ua.send_request(input_data)
		result[new_index] = http_ua.response_object.status
def run_requests(iterations):
	"""Post request with no content-type AND no content-length"""
	x = ruleset.Input(method="GET", protocol="http",port=80,uri='/?X="><script>alert(1);</script>',dest_addr="localhost",headers={"Host":"localhost","User-Agent":"ModSecurity CRS 3 test"})
	import threading
	returns = {}
	threads = []
	for i in range(5):
		t = threading.Thread(target=send_requests,args=(x,100, returns,i,))
		threads.append(t)
		t.start()
	for t in threads:
		t.join()
	status_not_403 = 0
	status_403 = 0
	for status in returns.values():
		if status == 403:	
			status_403 += 1
		else:
			status_not_403 += 1
	x = (status_403/(len(returns)*1.0))*100
	y = (status_not_403/(len(returns)*1.0))*100
	print "403s =", x
	print "not 403s =", y
	return (x,y)
		
def test_sampling():
	print "running"
	block,passed = run_requests(100)
	assert block < 55 and block > 45



try:
    import ConfigParser as configparser
except ImportError:
    import configparser
import os
import pytest


def pytest_addoption(parser):
    parser.addoption('--config', action='store', default='modsec2-apache')


@pytest.fixture(scope='session')
def config(request):
    cp = configparser.RawConfigParser()
    cp.read(os.path.join(os.path.dirname(__file__), 'config.ini'))
    return dict(cp.items(request.config.getoption('--config')))
